<?php
/**
 * Request Cache on/off settings
 * @package WP-Favicons
 * @since 0.5.0
 * @author Edward de Leau <e@leau.net>, http://wp.leau.co
 * @copyright GPL 2
 */
namespace leau\co\wp_favicons;

if (!class_exists("cache_request_cache"))
{
	class cache_request_cache extends Plugin
	{

		/**
		 *
		 * Enter description here ...
		 */
		function AddFilter()
    	{
    		/* parent class arranges that this is not called when the bool is off */
    		Log::M('request cache filters added');
    		add_filter(Config::GetPluginSlug() . 'pre_http', array($this,'PreHttp'), 6, 2);
    		add_action(Config::GetPluginSlug() . 'post_http_full', array($this,'PostHttpFull'), 6, 2);
    		add_action(Config::GetPluginSlug() . 'post_http_head', array($this,'PostHttpHead'), 6, 2);
    		add_filter(Config::GetPluginSlug() . 'timebuffer', array($this,'Timebuffer'), 6, 1);
    	}

    	/**
    	 *
    	 * Enter description here ...
    	 * @param $favicon
    	 */
    	function Timebuffer($favicon)
    	{
    		global $wpdb;
			$table_name_2 = Database::TableName2();
			$state = $favicon->GetState();
			// 30 seconds between icons retrieval
			try {
				$row_uri = $wpdb->get_results(	"SELECT itime " .
					"FROM $table_name_2 WHERE unix_timestamp(itime) > unix_timestamp(now()) - 30;",
					ARRAY_A);
			} catch  (Exception $e) {
				Log::F($favicon,'Could not Get Response ' . $e->getMessage());
			}
			if (empty($row_uri) || !is_array($row_uri))
			{
				// no last entry in the last 60 seconds so go ahead
				$favicon->SetState(true);
				return $favicon;
			}
			else
			{
				// entry the last 60 seconds so dont do another
				$favicon->SetState(false);
				return $favicon;
			}
    	}

    	/**
    	 *
    	 * Enter description here ...
    	 * @param $favicon
    	 * @param $response
    	 */
    	function PreHttp($favicon, $response)
    	{
    		if ($dbrequest=$this->CheckRequest($favicon))
			{
				if ($dbrequest['request_response_code'])
				{
					$request = array();
					$response['response']['code'] = $dbrequest['request_response_code'];
					$response['headers'] = unserialize($dbrequest['request_headers']);
					$response['body'] = $dbrequest['request_body'];
				}
				elseif ($dbrequest['request_response_message'])
				{
					$response = new \WP_Error('http_request_failed',
						sprintf(__('%s'), $dbrequest['request_response_message']));
				}
			}
			return $response;
    	}

    	function PostHttpFull($favicon, $response)
    	{
    		$this->LogRequest($favicon, $response);
    	}

		function PostHttpHead($favicon, $response)
    	{
    		if (!( !is_wp_error( $response ) &&
			 (in_array($response['response']['code'], array('200','404','405')))))
			{
				$this->LogRequest($favicon, $response);
			}
    	}


		/**
		 *
		 * Enter description here ...
		 * @param $uri
		 * @param $response
		 */
		function LogRequest($favicon, $response)
		{
			global $wpdb;
			$table_name_2 = Database::TableName2();

			$uri = $favicon->LastUriAddedToUriArr();
			$response_message = '';
			$response_body = '';
			$response_headers = '';
			$response_code = '';
			$response_cookies = '';

			if (is_wp_error( $response ))
			{
				$response_message = $response->get_error_message();
			}
			else
			{
				if (array_key_exists('body',$response))
				{
					$response_body = $response['body'];
				}
				if (array_key_exists('headers',$response))
				{
					$response_headers = serialize($response['headers']);
				}
				if (array_key_exists('response',$response))
				{
					$subresponse = $response['response'];
					if (array_key_exists('code',$subresponse))
					{
						$response_code 	= $subresponse['code'];
					}
					if (array_key_exists('message',$subresponse))
					{
						$response_message 	= $subresponse['message'];
					}
				}
				if (array_key_exists('cookies',$response))
				{
					$response_cookies = serialize($response['cookies']);
				}
			}
			try {
				$wpdb->insert($table_name_2,array(
				'uri' 						=> $uri,
				'uri_hash' 					=> md5($uri),
				'request_headers'			=> $response_headers,
				'request_cookies'			=> $response_cookies,
				'request_response_code'		=> $response_code,
				'request_response_message' 	=> $response_message,
				'request_body' 				=> $response_body,
				));
			} catch (Exception $e) {
				Log::F($favicon,'Could not write Response for '.
					$e->getMessage());
			}
			Log::F($favicon,' written in Request Log');
		}


	   /**
		 * Check and returns historical requests from the Request Cache
		 * @param $uri
		 */
		function CheckRequest($favicon)
		{
			global $wpdb;
			$table_name_2 = Database::TableName2();
			$uri = $favicon->LastUriAddedToUriArr();

			try {
				$row_uri = $wpdb->get_results(	"SELECT * " .
					"FROM $table_name_2 ".
					"WHERE `uri_hash` = '" . md5($uri) . "';",
					ARRAY_A);
			} catch  (Exception $e) {
				Log::F($favicon,'Could not Get Response for '.
					$uri . " : " . $e->getMessage());
			}
			if (!empty($row_uri) && is_array($row_uri))
			{
				Log::F($favicon,'retrieved from Request Cache');
				return $row_uri[0];
			}
			else
			{
				Log::F($favicon,'Not yet in Request Cache');
				return false;
			}
		}




	}
}

