<?php
/**
 * Contains Classes for the Favicon Content Replacers.
 *
 * This file contains all the clases for actually replacing the
 * favicons in the content and the widgets.
 *
 * @package WP-Favicons
 * @since 0.1
 * @author Edward de Leau <e@leau.net>
 * @copyright Edward de Leau, http://edward.de.leau.net
 */
namespace leau\co\wp_favicons;
/**
 * Class: EdlLinkFaviconContent
 *
 * This class adds a favicon to a link based on the generic class EdlHtmlHref and in the first place based on the public google favicons
 * though one can add a physical icon in the file cache so that this one will take preference.
 *
 * this is a specific factory
 *
 * @package WP-Favicons
 * @since 0.1
 * @author Edward de Leau <e@leau.net>
 * @copyright Edward de Leau, http://edward.de.leau.net
 */

// note that fully quantified namespaces should be used
// when referencing a class from another scope
if (!class_exists("EdlLinkFaviconContent") && class_exists("leau\co\wp_favicons\EdlHtmlHref"))
{
	class EdlLinkFaviconContent extends EdlHtmlHref
	{
		public  $_filter = 'the_content';

		/**
		 * Method: constructor
		 * @since 0.1
		 */
		public function __construct($moduleID, $pluginID)
		{
			/* set the new filter in the object for reference */
			$this->_filter = Config::GetModulePluginVar($moduleID, $pluginID,
				'filter_context_name');

			/* determine mode:
			 * href = use href class
			 * text = put it before text
			 */
			if (Config::GetModulePluginVar($moduleID,$pluginID,'mode')=='href')
			{
				add_filter($this->_filter, array($this,'ReplaceAll'), 9);
			}
			else
			{
				// if needed by additional functionality
			}
			$admin_options = Config::GetOptionsAsArray();
			if (isset($admin_options[Config::GetPluginSlug() . 'exclude_filetypes'])) {
				parent::SetNotAllowedExtension($admin_options[Config::GetPluginSlug() .
				'exclude_filetypes']);
			}
		}

		/**
		 * (non-PHPdoc)
		 * @see edl-classes/EdlHtmlHref::SpecificHrefBefore()
		 */
		public function SpecificHrefBefore( /*string*/ $uri)
		{
			if (!$uri) { return ''; }
			$faviconFactory = new FaviconFactory(new Favicon($uri),$this->_filter);
			return $faviconFactory->GetFaviconAsHtml();
		}
	}
}